@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000,2007 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1996-2000,2007,2010,2013 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1999-2008 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
@@ -178,6 +178,8 @@ void mutt_free_body (BODY **p)
     }
 
     FREE (&b->filename);
+    FREE (&b->d_filename);
+    FREE (&b->charset);
     FREE (&b->content);
     FREE (&b->xtype);
     FREE (&b->subtype);
@@ -968,6 +970,9 @@ int mutt_check_overwrite (const char *attname, const char *path,
     if (directory)
     {
       switch (mutt_multi_choice
+      /* L10N:
+         Means "The path you specified as the destination file is a directory."
+         See the msgid "Save to file: " (alias.c, recvattach.c) */
 	      (_("File is a directory, save under it? [(y)es, (n)o, (a)ll]"), _("yna")))
       {
 	case 3:		/* all */
@@ -984,12 +989,15 @@ int mutt_check_overwrite (const char *attname, const char *path,
 	  return 1;
       }
     }
-    else if ((rc = mutt_yesorno (_("File is a directory, save under it?"), M_YES)) != M_YES)
-      return (rc == M_NO) ? 1 : -1;
+    /* L10N:
+       Means "The path you specified as the destination file is a directory."
+       See the msgid "Save to file: " (alias.c, recvattach.c) */
+    else if ((rc = mutt_yesorno (_("File is a directory, save under it?"), MUTT_YES)) != MUTT_YES)
+      return (rc == MUTT_NO) ? 1 : -1;
 
     strfcpy (tmp, mutt_basename (NONULL (attname)), sizeof (tmp));
     if (mutt_get_field (_("File under directory: "), tmp, sizeof (tmp),
-                                    M_FILE | M_CLEAR) != 0 || !tmp[0])
+                                    MUTT_FILE | MUTT_CLEAR) != 0 || !tmp[0])
       return (-1);
     mutt_concat_path (fname, path, tmp, flen);
   }
@@ -1005,10 +1013,10 @@ int mutt_check_overwrite (const char *attname, const char *path,
 	return 1;
 
       case 2: /* append */
-        *append = M_SAVE_APPEND;
+        *append = MUTT_SAVE_APPEND;
         break;
       case 1: /* overwrite */
-        *append = M_SAVE_OVERWRITE;
+        *append = MUTT_SAVE_OVERWRITE;
         break;
     }
   }
@@ -1047,6 +1055,7 @@ void mutt_safe_path (char *s, size_t l, ADDRESS *a)
 void mutt_FormatString (char *dest,		/* output buffer */
 			size_t destlen,		/* output buffer len */
 			size_t col,		/* starting column (nonzero when called recursively) */
+                        int cols,               /* maximum columns */
 			const char *src,	/* template string */
 			format_t *callback,	/* callback for processing */
 			unsigned long data,	/* callback data */
@@ -1062,10 +1071,10 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
   prefix[0] = '\0';
   destlen--; /* save room for the terminal \0 */
-  wlen = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
+  wlen = ((flags & MUTT_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
   col += wlen;
 
-  if ((flags & M_FORMAT_NOFILTER) == 0)
+  if ((flags & MUTT_FORMAT_NOFILTER) == 0)
   {
     int off = -1;
 
@@ -1111,8 +1120,8 @@ void mutt_FormatString (char *dest,		/* output buffer */
         mutt_extract_token(word, srcbuf, 0);
         dprint(3, (debugfile, "fmtpipe %2d: %s\n", i++, word->data));
         mutt_buffer_addch(command, '\'');
-        mutt_FormatString(buf, sizeof(buf), 0, word->data, callback, data,
-                          flags | M_FORMAT_NOFILTER);
+        mutt_FormatString(buf, sizeof(buf), 0, cols, word->data, callback, data,
+                          flags | MUTT_FORMAT_NOFILTER);
         for (p = buf; p && *p; p++)
         {
           if (*p == '\'')
@@ -1132,7 +1141,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
       col -= wlen;	/* reset to passed in value */
       wptr = dest;      /* reset write ptr */
-      wlen = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
+      wlen = ((flags & MUTT_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
       if ((pid = mutt_create_filter(command->data, NULL, &filter, NULL)) != -1)
       {
 	int rc;
@@ -1166,7 +1175,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 		 * it back for the recursive call since the expansion of
 		 * format pipes does not try to append a nul itself.
 		 */
-		mutt_FormatString(dest, destlen+1, col, recycler, callback, data, flags);
+		mutt_FormatString(dest, destlen+1, col, cols, recycler, callback, data, flags);
 		FREE(&recycler);
 	      }
 	    }
@@ -1207,12 +1216,12 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
       if (*src == '?')
       {
-	flags |= M_FORMAT_OPTIONAL;
+	flags |= MUTT_FORMAT_OPTIONAL;
 	src++;
       }
       else
       {
-	flags &= ~M_FORMAT_OPTIONAL;
+	flags &= ~MUTT_FORMAT_OPTIONAL;
 
 	/* eat the format string */
 	cp = prefix;
@@ -1231,7 +1240,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
       ch = *src++; /* save the character to switch on */
 
-      if (flags & M_FORMAT_OPTIONAL)
+      if (flags & MUTT_FORMAT_OPTIONAL)
       {
         if (*src != '?')
           break; /* bad format */
@@ -1276,23 +1285,35 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	  pl = pw = 1;
 
 	/* see if there's room to add content, else ignore */
-	if ((col < COLS && wlen < destlen) || soft)
+	if ((col < cols && wlen < destlen) || soft)
 	{
 	  int pad;
 
 	  /* get contents after padding */
-	  mutt_FormatString (buf, sizeof (buf), 0, src + pl, callback, data, flags);
+	  mutt_FormatString (buf, sizeof (buf), 0, cols, src + pl, callback, data, flags);
 	  len = mutt_strlen (buf);
 	  wid = mutt_strwidth (buf);
 
-	  /* try to consume as many columns as we can, if we don't have
-	   * memory for that, use as much memory as possible */
-	  pad = (COLS - col - wid) / pw;
-	  if (pad > 0 && wlen + (pad * pl) + len > destlen)
-	    pad = ((signed)(destlen - wlen - len)) / pl;
-	  if (pad > 0)
+	  pad = (cols - col - wid) / pw;
+	  if (pad >= 0)
 	  {
-	    while (pad--)
+            /* try to consume as many columns as we can, if we don't have
+             * memory for that, use as much memory as possible */
+            if (wlen + (pad * pl) + len > destlen)
+              pad = (destlen > wlen + len) ? ((destlen - wlen - len) / pl) : 0;
+            else
+            {
+              /* Add pre-spacing to make multi-column pad characters and
+               * the contents after padding line up */
+              while ((col + (pad * pw) + wid < cols) &&
+                     (wlen + (pad * pl) + len < destlen))
+              {
+                *wptr++ = ' ';
+                wlen++;
+                col++;
+              }
+            }
+	    while (pad-- > 0)
 	    {
 	      memcpy (wptr, src, pl);
 	      wptr += pl;
@@ -1302,17 +1323,26 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	  }
 	  else if (soft && pad < 0)
 	  {
-	    int offset = ((flags & M_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
+	    int offset = ((flags & MUTT_FORMAT_ARROWCURSOR) && option (OPTARROWCURSOR)) ? 3 : 0;
+            int avail_cols = (cols > offset) ? (cols - offset) : 0;
 	    /* \0-terminate dest for length computation in mutt_wstr_trunc() */
 	    *wptr = 0;
 	    /* make sure right part is at most as wide as display */
-	    len = mutt_wstr_trunc (buf, destlen, COLS-offset, &wid);
+	    len = mutt_wstr_trunc (buf, destlen, avail_cols, &wid);
 	    /* truncate left so that right part fits completely in */
-	    wlen = mutt_wstr_trunc (dest, destlen - len, col + pad*pw -offset, &col);
+	    wlen = mutt_wstr_trunc (dest, destlen - len, avail_cols - wid, &col);
 	    wptr = dest + wlen;
+            /* Multi-column characters may be truncated in the middle.
+             * Add spacing so the right hand side lines up. */
+            while ((col + wid < avail_cols) && (wlen + len < destlen))
+            {
+              *wptr++ = ' ';
+              wlen++;
+              col++;
+            }
 	  }
 	  if (len + wlen > destlen)
-	    len = mutt_wstr_trunc (buf, destlen - wlen, COLS - col, NULL);
+	    len = mutt_wstr_trunc (buf, destlen - wlen, cols - col, NULL);
 	  memcpy (wptr, buf, len);
 	  wptr += len;
 	  wlen += len;
@@ -1329,9 +1359,9 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	  pl = pw = 1;
 
 	/* see if there's room to add content, else ignore */
-	if (col < COLS && wlen < destlen)
+	if (col < cols && wlen < destlen)
 	{
-	  c = (COLS - col) / pw;
+	  c = (cols - col) / pw;
 	  if (c > 0 && wlen + (c * pl) > destlen)
 	    c = ((signed)(destlen - wlen)) / pl;
 	  while (c > 0)
@@ -1362,7 +1392,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	}
 	
 	/* use callback function to handle this case */
-	src = callback (buf, sizeof (buf), col, ch, src, prefix, ifstring, elsestring, data, flags);
+	src = callback (buf, sizeof (buf), col, cols, ch, src, prefix, ifstring, elsestring, data, flags);
 
 	if (tolower)
 	  mutt_strlower (buf);
@@ -1375,7 +1405,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	}
 	
 	if ((len = mutt_strlen (buf)) + wlen > destlen)
-	  len = mutt_wstr_trunc (buf, destlen - wlen, COLS - col, NULL);
+	  len = mutt_wstr_trunc (buf, destlen - wlen, cols - col, NULL);
 
 	memcpy (wptr, buf, len);
 	wptr += len;
@@ -1437,13 +1467,13 @@ void mutt_FormatString (char *dest,		/* output buffer */
   *wptr = 0;
 
 #if 0
-  if (flags & M_FORMAT_MAKEPRINT)
+  if (flags & MUTT_FORMAT_MAKEPRINT)
   {
     /* Make sure that the string is printable by changing all non-printable
        chars to dots, or spaces for non-printable whitespace */
     for (cp = dest ; *cp ; cp++)
       if (!IsPrint (*cp) &&
-	  !((flags & M_FORMAT_TREE) && (*cp <= M_TREE_MAX)))
+	  !((flags & MUTT_FORMAT_TREE) && (*cp <= MUTT_TREE_MAX)))
 	*cp = isspace ((unsigned char) *cp) ? ' ' : '.';
   }
 #endif
@@ -1496,7 +1526,7 @@ int mutt_save_confirm (const char *s, struct stat *st)
   magic = mx_get_magic (s);
 
 #ifdef USE_POP
-  if (magic == M_POP)
+  if (magic == MUTT_POP)
   {
     mutt_error _("Can't save message to POP mailbox.");
     return 1;
@@ -1508,7 +1538,7 @@ int mutt_save_confirm (const char *s, struct stat *st)
     if (option (OPTCONFIRMAPPEND))
     {
       snprintf (tmp, sizeof (tmp), _("Append messages to %s?"), s);
-      if ((rc = mutt_yesorno (tmp, M_YES)) == M_NO)
+      if ((rc = mutt_yesorno (tmp, MUTT_YES)) == MUTT_NO)
 	ret = 1;
       else if (rc == -1)
 	ret = -1;
@@ -1523,7 +1553,7 @@ int mutt_save_confirm (const char *s, struct stat *st)
       return 1;
     }
   }
-  else if (magic != M_IMAP)
+  else if (magic != MUTT_IMAP)
   {
     st->st_mtime = 0;
     st->st_atime = 0;
@@ -1533,7 +1563,7 @@ int mutt_save_confirm (const char *s, struct stat *st)
       if (option (OPTCONFIRMCREATE))
       {
 	snprintf (tmp, sizeof (tmp), _("Create %s?"), s);
-	if ((rc = mutt_yesorno (tmp, M_YES)) == M_NO)
+	if ((rc = mutt_yesorno (tmp, MUTT_YES)) == MUTT_NO)
 	  ret = 1;
 	else if (rc == -1)
 	  ret = -1;
@@ -1546,13 +1576,13 @@ int mutt_save_confirm (const char *s, struct stat *st)
     }
   }
 
-  CLEARLINE (LINES-1);
+  mutt_window_clearline (MuttMessageWindow, 0);
   return (ret);
 }
 
 void state_prefix_putc (char c, STATE *s)
 {
-  if (s->flags & M_PENDINGPREFIX)
+  if (s->flags & MUTT_PENDINGPREFIX)
   {
     state_reset_prefix (s);
     if (s->prefix)
@@ -1579,7 +1609,7 @@ int state_printf (STATE *s, const char *fmt, ...)
 
 void state_mark_attach (STATE *s)
 {
-  if ((s->flags & M_DISPLAY) && !mutt_strcmp (Pager, "builtin"))
+  if ((s->flags & MUTT_DISPLAY) && !mutt_strcmp (Pager, "builtin"))
     state_puts (AttachmentMarker, s);
 }
 
