@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2002 Vsevolod Volkov <vvv@mutt.org.ua>
- * Copyright (C) 2006-7 Rocco Rutte <pdmef@gmx.net>
+ * Copyright (C) 2006-2007,2009 Rocco Rutte <pdmef@gmx.net>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
 
 #ifdef USE_HCACHE
 #define HC_FNAME	"mutt"		/* filename for hcache as POP lacks paths */
@@ -55,7 +56,7 @@ static int fetch_message (char *line, void *file)
  * Read header
  * returns:
  *  0 on success
- * -1 - conection lost,
+ * -1 - connection lost,
  * -2 - invalid command or execution error,
  * -3 - error writing to tempfile
  */
@@ -141,8 +142,16 @@ static int fetch_uidl (char *line, void *data)
   int i, index;
   CONTEXT *ctx = (CONTEXT *)data;
   POP_DATA *pop_data = (POP_DATA *)ctx->data;
+  char *endp;
+
+  errno = 0;
+  index = strtol(line, &endp, 10);
+  if (errno)
+      return -1;
+  while (*endp == ' ')
+      endp++;
+  memmove(line, endp, strlen(endp) + 1);
 
-  sscanf (line, "%d %s", &index, line);
   for (i = 0; i < ctx->msgcount; i++)
     if (!mutt_strcmp (line, ctx->hdrs[i]->data))
       break;
@@ -220,7 +229,7 @@ static header_cache_t *pop_hcache_open (POP_DATA *pop_data, const char *path)
  * Read headers
  * returns:
  *  0 on success
- * -1 - conection lost,
+ * -1 - connection lost,
  * -2 - invalid command or execution error,
  * -3 - error writing to tempfile
  */
@@ -270,7 +279,7 @@ static int pop_fetch_headers (CONTEXT *ctx)
 
   if (!ctx->quiet)
     mutt_progress_init (&progress, _("Fetching message headers..."),
-                        M_PROGRESS_MSG, ReadInc, new_count - old_count);
+                        MUTT_PROGRESS_MSG, ReadInc, new_count - old_count);
 
   if (ret == 0)
   {
@@ -324,7 +333,7 @@ static int pop_fetch_headers (CONTEXT *ctx)
 #if USE_HCACHE
       else
       {
-	mutt_hcache_store (hc, ctx->hdrs[i]->data, ctx->hdrs[i], 0, strlen);
+	mutt_hcache_store (hc, ctx->hdrs[i]->data, ctx->hdrs[i], 0, strlen, MUTT_GENERATE_UIDVALIDITY);
       }
 
       FREE(&data);
@@ -388,7 +397,7 @@ static int pop_fetch_headers (CONTEXT *ctx)
 }
 
 /* open POP mailbox - fetch only headers */
-int pop_open_mailbox (CONTEXT *ctx)
+static int pop_open_mailbox (CONTEXT *ctx)
 {
   int ret;
   char buf[LONG_STRING];
@@ -412,12 +421,13 @@ int pop_open_mailbox (CONTEXT *ctx)
     return -1;
 
   FREE (&ctx->path);
+  FREE (&ctx->realpath);
   ctx->path = safe_strdup (buf);
+  ctx->realpath = safe_strdup (ctx->path);
 
   pop_data = safe_calloc (1, sizeof (POP_DATA));
   pop_data->conn = conn;
   ctx->data = pop_data;
-  ctx->mx_close = pop_close_mailbox;
 
   if (pop_open_connection (pop_data) < 0)
     return -1;
@@ -427,12 +437,12 @@ int pop_open_mailbox (CONTEXT *ctx)
 
   /* init (hard-coded) ACL rights */
   memset (ctx->rights, 0, sizeof (ctx->rights));
-  mutt_bit_set (ctx->rights, M_ACL_SEEN);
-  mutt_bit_set (ctx->rights, M_ACL_DELETE);
+  mutt_bit_set (ctx->rights, MUTT_ACL_SEEN);
+  mutt_bit_set (ctx->rights, MUTT_ACL_DELETE);
 #if USE_HCACHE
   /* flags are managed using header cache, so it only makes sense to
    * enable them in that case */
-  mutt_bit_set (ctx->rights, M_ACL_WRITE);
+  mutt_bit_set (ctx->rights, MUTT_ACL_WRITE);
 #endif
 
   FOREVER
@@ -504,7 +514,7 @@ int pop_close_mailbox (CONTEXT *ctx)
 }
 
 /* fetch message from POP server */
-int pop_fetch_message (MESSAGE* msg, CONTEXT* ctx, int msgno)
+static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
 {
   int ret;
   void *uidl;
@@ -561,7 +571,7 @@ int pop_fetch_message (MESSAGE* msg, CONTEXT* ctx, int msgno)
     }
 
     mutt_progress_init (&progressbar, _("Fetching message..."),
-			M_PROGRESS_SIZE, NetInc, h->content->length + h->content->offset - 1);
+			MUTT_PROGRESS_SIZE, NetInc, h->content->length + h->content->offset - 1);
 
     /* see if we can put in body cache; use our cache as fallback */
     if (!(msg->fp = mutt_bcache_put (pop_data->bcache, h->data, 1)))
@@ -618,8 +628,15 @@ int pop_fetch_message (MESSAGE* msg, CONTEXT* ctx, int msgno)
   }
   rewind (msg->fp);
   uidl = h->data;
+
+  /* we replace envelop, key in subj_hash has to be updated as well */
+  if (ctx->subj_hash && h->env->real_subj)
+    hash_delete (ctx->subj_hash, h->env->real_subj, h, NULL);
   mutt_free_envelope (&h->env);
   h->env = mutt_read_rfc822_header (msg->fp, h, 0, 0);
+  if (ctx->subj_hash && h->env->real_subj)
+    hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
+
   h->data = uidl;
   h->lines = 0;
   fgets (buf, sizeof (buf), msg->fp);
@@ -641,6 +658,11 @@ int pop_fetch_message (MESSAGE* msg, CONTEXT* ctx, int msgno)
   return 0;
 }
 
+static int pop_close_message (CONTEXT *ctx, MESSAGE *msg)
+{
+  return safe_fclose (&msg->fp);
+}
+
 /* update POP mailbox - delete messages from server */
 int pop_sync_mailbox (CONTEXT *ctx, int *index_hint)
 {
@@ -660,7 +682,7 @@ int pop_sync_mailbox (CONTEXT *ctx, int *index_hint)
       return -1;
 
     mutt_progress_init (&progress, _("Marking messages deleted..."),
-			M_PROGRESS_MSG, WriteInc, ctx->deleted);
+			MUTT_PROGRESS_MSG, WriteInc, ctx->deleted);
 
 #if USE_HCACHE
     hc = pop_hcache_open (pop_data, ctx->path);
@@ -686,7 +708,7 @@ int pop_sync_mailbox (CONTEXT *ctx, int *index_hint)
 #if USE_HCACHE
       if (ctx->hdrs[i]->changed)
       {
-	mutt_hcache_store (hc, ctx->hdrs[i]->data, ctx->hdrs[i], 0, strlen);
+	mutt_hcache_store (hc, ctx->hdrs[i]->data, ctx->hdrs[i], 0, strlen, MUTT_GENERATE_UIDVALIDITY);
       }
 #endif
 
@@ -720,7 +742,7 @@ int pop_sync_mailbox (CONTEXT *ctx, int *index_hint)
 }
 
 /* Check for new messages and fetch headers */
-int pop_check_mailbox (CONTEXT *ctx, int *index_hint)
+static int pop_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   int ret;
   POP_DATA *pop_data = (POP_DATA *)ctx->data;
@@ -746,7 +768,7 @@ int pop_check_mailbox (CONTEXT *ctx, int *index_hint)
     return -1;
 
   if (ret > 0)
-    return M_NEW_MAIL;
+    return MUTT_NEW_MAIL;
 
   return 0;
 }
@@ -834,7 +856,7 @@ void pop_fetch_mail (void)
     goto finish;
   }
 
-  if (mx_open_mailbox (NONULL (Spoolfile), M_APPEND, &ctx) == NULL)
+  if (mx_open_mailbox (NONULL (Spoolfile), MUTT_APPEND, &ctx) == NULL)
     goto finish;
 
   delanswer = query_quadoption (OPT_POPDELETE, _("Delete messages from server?"));
@@ -844,7 +866,7 @@ void pop_fetch_mail (void)
 
   for (i = last + 1 ; i <= msgs ; i++)
   {
-    if ((msg = mx_open_new_message (&ctx, NULL, M_ADD_FROM)) == NULL)
+    if ((msg = mx_open_new_message (&ctx, NULL, MUTT_ADD_FROM)) == NULL)
       ret = -3;
     else
     {
@@ -859,10 +881,10 @@ void pop_fetch_mail (void)
 	ret = -3;
       }
 
-      mx_close_message (&msg);
+      mx_close_message (&ctx, &msg);
     }
 
-    if (ret == 0 && delanswer == M_YES)
+    if (ret == 0 && delanswer == MUTT_YES)
     {
       /* delete the message on the server */
       snprintf (buffer, sizeof (buffer), "DELE %d\r\n", i);
@@ -912,3 +934,14 @@ fail:
   mutt_socket_close (conn);
   FREE (&pop_data);
 }
+
+struct mx_ops mx_pop_ops = {
+  .open = pop_open_mailbox,
+  .open_append = NULL,
+  .close = pop_close_mailbox,
+  .open_msg = pop_fetch_message,
+  .close_msg = pop_close_message,
+  .check = pop_check_mailbox,
+  .commit_msg = NULL,
+  .open_new_msg = NULL,
+};
